﻿' (C) Copyright 2010 by Autodesk, Inc. 
'
' Permission to use, copy, modify, and distribute this software in
' object code form for any purpose and without fee is hereby granted,
' provided that the above copyright notice appears in all copies and
' that both that copyright notice and the limited warranty and
' restricted rights notice below appear in all supporting
' documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
' UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is subject
' to restrictions set forth in FAR 52.227-19 (Commercial Computer
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
' (Rights in Technical Data and Computer Software), as applicable.

Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput


Namespace Parametric_API_Demo_VB


  Public Class MyCommands

    <CommandMethod("TESTFIXED")> _
    Public Shared Sub testFixedCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Using myT As Transaction = tm.StartTransaction()
        'Create DB resident line
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        Dim entity As Entity = New Line(New Point3d(12, 5, 0), New Point3d(15, 12, 0))
        btr.AppendEntity(entity)
        myT.AddNewlyCreatedDBObject(entity, True)

        'To query the subentities of the line, we create and use a protocol extension (PE) provided by the associativity API
        Dim subentityIdPE As AssocPersSubentityIdPE
        Dim peCls As RXClass = AssocPersSubentityIdPE.GetClass(GetType(AssocPersSubentityIdPE))
        Dim pSubentityIdPE As IntPtr = entity.QueryX(peCls)
        If pSubentityIdPE = IntPtr.Zero Then
          System.Windows.MessageBox.Show("cannot get pSubentityIdPE")
          Exit Sub
        End If
        subentityIdPE = TryCast(AssocPersSubentityIdPE.Create(pSubentityIdPE, False), AssocPersSubentityIdPE)
        If subentityIdPE Is Nothing Then
          System.Windows.MessageBox.Show("cannot get subentityIdPE")
          Exit Sub
        End If
        'Now we have the PE, we query the subentities
        Dim edgeSubentityIds As SubentityId() = Nothing
        'First we retrieve a list of all edges (a line has one edge)
        edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge)
        'Now we retrieve the vertices associated with the first edge in our array.
        'In this case we have one edge, and the edge has three vertices - start, end and middle.
        Dim startSID As SubentityId = SubentityId.Null, endSID As SubentityId = SubentityId.Null
        Dim other As SubentityId() = Nothing
        subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds(0), startSID, endSID, other)
        'The PE returns a SubEntId. We want a FullSubentityPath
        Dim subentPathEdge As FullSubentityPath, subentPath1 As FullSubentityPath
        subentPathEdge = New FullSubentityPath(New ObjectId(0) {entity.ObjectId}, edgeSubentityIds(0)) 'The line edge
        subentPath1 = New FullSubentityPath(New ObjectId(0) {entity.ObjectId}, startSID) 'The edge's startpoint.

        'We call a helper function to retrieve or create a constraints group
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          'Pass in geometry to constrain (the line edge)
          Dim consGeom As ConstrainedGeometry = constGrp.AddConstrainedGeometry(subentPathEdge)
          'Now create the constraint, a Fixed constraint applied to the line's startpoint.
          Dim paths As FullSubentityPath() = New FullSubentityPath(0) {subentPath1}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Fix, paths)
        End Using

        myT.Commit()
      End Using
    End Sub


    'horizontal constraint
    <CommandMethod("TESTHORIZONTAL")> _
    Public Shared Sub testHorizontalCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Using myT As Transaction = tm.StartTransaction()
        'Add line to database
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        Dim line As New Line(New Point3d(5, 5, 0), New Point3d(15, 15, 0))
        btr.AppendEntity(line)
        myT.AddNewlyCreatedDBObject(line, True)

        'Our constrained geometry will be the edge of the line
        Dim subentPath As FullSubentityPath = GetFullSubentityPath(line, SubentityType.Edge)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          'Add the constrained geometry to which the geometrical constraint will be applied
          consGeom = constGrp.AddConstrainedGeometry(subentPath)
          Dim paths As FullSubentityPath() = New FullSubentityPath(0) {subentPath}
          'Now add the geometrical constraint
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Horizontal, paths)
          Dim constraintArray As New List(Of GeometricalConstraint)()
          constraintArray.Add(newConstraint)
        End Using

        myT.Commit()
      End Using
    End Sub

    'horizontal constraint between 2 DBPoints
    <CommandMethod("TESTHORIZONTALPOINTS")> _
    Public Shared Sub testHorizontalPointsCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Using myT As Transaction = tm.StartTransaction()
        ' Add Point entities to the database (change PDMODE variable in AutoCAD to see them better (e.g. PDMODE=3))
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        Dim point3d1 As New Point3d(10, 10, 0)
        Dim point3d2 As New Point3d(20, 20, 0)
        Dim dimPt As New Point3d(36, 20, 0)
        Dim point1 As New DBPoint(point3d1)
        Dim point2 As New DBPoint(point3d2)
        btr.AppendEntity(point1)
        btr.AppendEntity(point2)
        myT.AddNewlyCreatedDBObject(point1, True)
        myT.AddNewlyCreatedDBObject(point2, True)

        'Our constrained geometry will be the vertex of each POINT
        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(point1, SubentityType.Vertex)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(point2, SubentityType.Vertex)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          'Add the geometrical constraint to be applied to the constrained geometry
          consGeom = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom = constGrp.AddConstrainedGeometry(subentPath2)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          'Now add geometrical constraints
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Horizontal, paths)
        End Using

        myT.Commit()
      End Using
    End Sub


    'horizontal constraint between start and end points of line
    <CommandMethod("TESTHORIZONTALALIGN")> _
    Public Shared Sub testHorizontalAlignCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Using myT As Transaction = tm.StartTransaction()
        'Create DB resident line
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        Dim entity As Entity = New Line(New Point3d(12, 5, 0), New Point3d(15, 12, 0))
        btr.AppendEntity(entity)
        myT.AddNewlyCreatedDBObject(entity, True)

        Dim subentPathEdge As FullSubentityPath, subentPath1 As FullSubentityPath, subentPath2 As FullSubentityPath

        'To query the subentities of the line, we create and use a protocol extension (PE) provided by the associativity API
        Dim subentityIdPE As AssocPersSubentityIdPE
        Dim peCls As RXClass = AssocPersSubentityIdPE.GetClass(GetType(AssocPersSubentityIdPE))
        Dim pSubentityIdPE As IntPtr = entity.QueryX(peCls)
        If pSubentityIdPE = IntPtr.Zero Then
          System.Windows.MessageBox.Show("cannot get pSubentityIdPE")
          Return
        End If
        subentityIdPE = TryCast(AssocPersSubentityIdPE.Create(pSubentityIdPE, False), AssocPersSubentityIdPE)
        If subentityIdPE Is Nothing Then
          System.Windows.MessageBox.Show("cannot get subentityIdPE")
          Return
        End If
        'Now we have the PE, we query the subentities
        Dim edgeSubentityIds As SubentityId() = Nothing
        'First we retrieve a list of all edges (a line has one edge)
        edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge)
        Dim startSID As SubentityId = SubentityId.Null, endSID As SubentityId = SubentityId.Null
        Dim other As SubentityId() = Nothing
        'Now we retrieve the vertices associated with that edge.
        subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds(0), startSID, endSID, other)
        'The PE returns a SubEntId. We want a Full SubentityPath
        subentPathEdge = New FullSubentityPath(New ObjectId(0) {entity.ObjectId}, edgeSubentityIds(0))
        subentPath1 = New FullSubentityPath(New ObjectId(0) {entity.ObjectId}, startSID)
        subentPath2 = New FullSubentityPath(New ObjectId(0) {entity.ObjectId}, endSID)

        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          'Pass in geometry to constrain (the line edge)
          consGeom = constGrp.AddConstrainedGeometry(subentPathEdge)
          'Now create the constraint (using the start and end vertices of the line)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Horizontal, paths)
        End Using

        myT.Commit()
      End Using
    End Sub







    'vertical constraint
    <CommandMethod("TESTVERTICAL")> _
    Public Shared Sub testVerticalCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim line As Line
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        line = New Line(New Point3d(10, 5, 0), New Point3d(7, 15, 0))
        btr.AppendEntity(line)
        myT.AddNewlyCreatedDBObject(line, True)

        ' We've put the protocol extension code into a helper function
        Dim subentPath As FullSubentityPath = GetFullSubentityPath(line, SubentityType.Edge)

        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom = constGrp.AddConstrainedGeometry(subentPath)
          Dim paths As FullSubentityPath() = New FullSubentityPath(0) {subentPath}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Vertical, paths)
        End Using

        myT.Commit()
      End Using
    End Sub



    'vertical constraint between start/end points of line
    <CommandMethod("TESTVERTICALALIGN")> _
    Public Shared Sub testVerticalAlignCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim entity As Entity
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        entity = New Line(New Point3d(12, 9, 0), New Point3d(15, 6, 0))
        btr.AppendEntity(entity)
        myT.AddNewlyCreatedDBObject(entity, True)

        Dim subentPathEdge As FullSubentityPath, subentPath1 As FullSubentityPath, subentPath2 As FullSubentityPath
        Dim subentityIdPE As AssocPersSubentityIdPE
        Dim peCls As RXClass = AssocPersSubentityIdPE.GetClass(GetType(AssocPersSubentityIdPE))
        Dim pSubentityIdPE As IntPtr = entity.QueryX(peCls)
        If pSubentityIdPE = IntPtr.Zero Then
          System.Windows.MessageBox.Show("cannot get pSubentityIdPE")
          Return
        End If
        subentityIdPE = TryCast(AssocPersSubentityIdPE.Create(pSubentityIdPE, False), AssocPersSubentityIdPE)
        If subentityIdPE Is Nothing Then
          System.Windows.MessageBox.Show("cannot get subentityIdPE")
          Return
        End If
        Dim edgeSubentityIds As SubentityId() = Nothing
        edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge)
        Dim startSID As SubentityId = SubentityId.Null, endSID As SubentityId = SubentityId.Null
        Dim other As SubentityId() = Nothing
        subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds(0), startSID, endSID, other)
        subentPathEdge = New FullSubentityPath(New ObjectId(0) {entity.ObjectId}, edgeSubentityIds(0))
        subentPath1 = New FullSubentityPath(New ObjectId(0) {entity.ObjectId}, startSID)
        subentPath2 = New FullSubentityPath(New ObjectId(0) {entity.ObjectId}, endSID)

        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom = constGrp.AddConstrainedGeometry(subentPathEdge)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Vertical, paths)
        End Using

        myT.Commit()
      End Using
    End Sub



    'coincident constraint between start points of 2 lines
    <CommandMethod("TESTCOINCIDENT")> _
    Public Shared Sub testCoincidentCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim line1 As Line, line2 As Line
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        line1 = New Line(New Point3d(18, 2, 0), New Point3d(12, 2, 0))
        line2 = New Line(New Point3d(2, 15, 0), New Point3d(7, 9, 0))
        btr.AppendEntity(line1)
        myT.AddNewlyCreatedDBObject(line1, True)
        btr.AppendEntity(line2)
        myT.AddNewlyCreatedDBObject(line2, True)

        Dim subentEdgePath1 As FullSubentityPath
        Dim subentEdgePath2 As FullSubentityPath
        Dim subentPath1 As FullSubentityPath
        Dim subentPath2 As FullSubentityPath
        Using entity As Entity = DirectCast(myT.GetObject(line1.ObjectId, OpenMode.ForRead, False), Entity)
          If entity Is Nothing Then
            System.Windows.MessageBox.Show("cannot get entity1")
            Return
          End If
          Dim subentityIdPE As AssocPersSubentityIdPE
          Dim peCls As RXClass = AssocPersSubentityIdPE.GetClass(GetType(AssocPersSubentityIdPE))
          Dim pSubentityIdPE As IntPtr = entity.QueryX(peCls)
          If pSubentityIdPE = IntPtr.Zero Then
            System.Windows.MessageBox.Show("cannot get pSubentityIdPE1")
            Return
          End If
          subentityIdPE = TryCast(AssocPersSubentityIdPE.Create(pSubentityIdPE, False), AssocPersSubentityIdPE)
          If subentityIdPE Is Nothing Then
            System.Windows.MessageBox.Show("cannot get subentityIdPE1")
            Return
          End If
          Dim edgeSubentityIds As SubentityId() = Nothing
          edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge)
          Dim startSID As SubentityId = SubentityId.Null, endSID As SubentityId = SubentityId.Null
          Dim other As SubentityId() = Nothing
          subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds(0), startSID, endSID, other)
          subentEdgePath1 = New FullSubentityPath(New ObjectId(0) {line1.ObjectId}, edgeSubentityIds(0))
          subentPath1 = New FullSubentityPath(New ObjectId(0) {line1.ObjectId}, startSID)
        End Using
        Using entity As Entity = DirectCast(myT.GetObject(line2.ObjectId, OpenMode.ForRead, False), Entity)
          If entity Is Nothing Then
            System.Windows.MessageBox.Show("cannot get entity2")
            Return
          End If
          ' Strictly speaking, we don't need to query the second line for its PE, because we've already created
          '  a PE for a line in the code above. But we would have to do this second entity is of a different type,
          '  which is why we're doing it in this sample code.
          Dim subentityIdPE As AssocPersSubentityIdPE
          Dim peCls As RXClass = AssocPersSubentityIdPE.GetClass(GetType(AssocPersSubentityIdPE))
          Dim pSubentityIdPE As IntPtr = entity.QueryX(peCls)
          If pSubentityIdPE = IntPtr.Zero Then
            System.Windows.MessageBox.Show("cannot get pSubentityIdPE2")
            Return
          End If
          subentityIdPE = TryCast(AssocPersSubentityIdPE.Create(pSubentityIdPE, False), AssocPersSubentityIdPE)
          If subentityIdPE Is Nothing Then
            System.Windows.MessageBox.Show("cannot get subentityIdPE2")
            Return
          End If
          Dim edgeSubentityIds As SubentityId() = Nothing
          edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge)
          Dim startSID As SubentityId = SubentityId.Null, endSID As SubentityId = SubentityId.Null
          Dim other As SubentityId() = Nothing
          subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds(0), startSID, endSID, other)
          subentEdgePath2 = New FullSubentityPath(New ObjectId(0) {line2.ObjectId}, edgeSubentityIds(0))
          subentPath2 = New FullSubentityPath(New ObjectId(0) {line2.ObjectId}, startSID)
        End Using
        Dim consGrpId As ObjectId = GetConstraintGroup(True)

        Dim consGeomEdge1 As ConstrainedGeometry = Nothing
        Dim consGeomEdge2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeomEdge1 = constGrp.AddConstrainedGeometry(subentEdgePath1)
          consGeomEdge2 = constGrp.AddConstrainedGeometry(subentEdgePath2)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Coincident, paths)
        End Using

        myT.Commit()
      End Using
    End Sub



    'colinear constraint between 2 lines
    <CommandMethod("TESTCOLINEAR")> _
    Public Shared Sub testColinearCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim line1 As Line, line2 As Line
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        line1 = New Line(New Point3d(33, 24, 0), New Point3d(19, 10, 0))
        line2 = New Line(New Point3d(22, 27, 0), New Point3d(18, 9, 0))
        btr.AppendEntity(line1)
        myT.AddNewlyCreatedDBObject(line1, True)
        btr.AppendEntity(line2)
        myT.AddNewlyCreatedDBObject(line2, True)

        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(line1, SubentityType.Edge)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(line2, SubentityType.Edge)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Collinear, paths)
        End Using

        myT.Commit()
      End Using
    End Sub




    'equal length constraint between 2 lines
    <CommandMethod("TESTEQUALLENGTH")> _
    Public Shared Sub testEqualLengthCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim line1 As Line, line2 As Line
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        line1 = New Line(New Point3d(23, 24, 0), New Point3d(56, 10, 0))
        line2 = New Line(New Point3d(12, 27, 0), New Point3d(29, 9, 0))
        btr.AppendEntity(line1)
        myT.AddNewlyCreatedDBObject(line1, True)
        btr.AppendEntity(line2)
        myT.AddNewlyCreatedDBObject(line2, True)

        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(line1, SubentityType.Edge)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(line2, SubentityType.Edge)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.EqualLength, paths)
        End Using

        myT.Commit()
      End Using
    End Sub



    'concentric constraint for 2 circles
    <CommandMethod("TESTCONCENTRIC")> _
    Public Shared Sub testConcentricCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim circle1 As Circle, circle2 As Circle
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        circle1 = New Circle(New Point3d(12, 24, 0), New Vector3d(0, 0, 1), 4)
        circle2 = New Circle(New Point3d(23, 14, 0), New Vector3d(0, 0, 1), 8)
        btr.AppendEntity(circle1)
        myT.AddNewlyCreatedDBObject(circle1, True)
        btr.AppendEntity(circle2)
        myT.AddNewlyCreatedDBObject(circle2, True)

        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(circle1, SubentityType.Edge)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(circle2, SubentityType.Edge)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Concentric, paths)
        End Using

        myT.Commit()
      End Using
    End Sub



    'equal radius for 2 circles
    <CommandMethod("TESTEQUALRADIUS")> _
    Public Shared Sub testEqualRadiusCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim circle1 As Circle, circle2 As Circle
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        circle1 = New Circle(New Point3d(15, 14, 0), New Vector3d(0, 0, 1), 4)
        circle2 = New Circle(New Point3d(15, 24, 0), New Vector3d(0, 0, 1), 6)
        btr.AppendEntity(circle1)
        myT.AddNewlyCreatedDBObject(circle1, True)
        btr.AppendEntity(circle2)
        myT.AddNewlyCreatedDBObject(circle2, True)

        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(circle1, SubentityType.Edge)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(circle2, SubentityType.Edge)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.EqualRadius, paths)
        End Using

        myT.Commit()
      End Using
    End Sub

    'parallel between 2 lines
    <CommandMethod("TESTPARALLEL")> _
    Public Shared Sub testParallelCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim line1 As Line, line2 As Line
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        line1 = New Line(New Point3d(12, 12, 0), New Point3d(34, 10, 0))
        line2 = New Line(New Point3d(15, 18, 0), New Point3d(19, 3, 0))
        btr.AppendEntity(line1)
        myT.AddNewlyCreatedDBObject(line1, True)
        btr.AppendEntity(line2)
        myT.AddNewlyCreatedDBObject(line2, True)

        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(line1, SubentityType.Edge)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(line2, SubentityType.Edge)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Parallel, paths)
        End Using

        myT.Commit()
      End Using
    End Sub





    'perpendicular between 2 lines
    <CommandMethod("TESTPERPENDICULAR")> _
    Public Shared Sub testPerpendicularCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim line1 As Line, line2 As Line
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        line1 = New Line(New Point3d(28, 0, 0), New Point3d(12, 19, 0))
        line2 = New Line(New Point3d(12, 1, 0), New Point3d(12, 28, 0))
        btr.AppendEntity(line1)
        myT.AddNewlyCreatedDBObject(line1, True)
        btr.AppendEntity(line2)
        myT.AddNewlyCreatedDBObject(line2, True)

        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(line1, SubentityType.Edge)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(line2, SubentityType.Edge)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Perpendicular, paths)
        End Using

        myT.Commit()
      End Using
    End Sub

    'G2smooth constraint between two splines
    <CommandMethod("TESTSMOOTH")> _
    Public Shared Sub testSmoothCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim spline1 As Spline, spline2 As Spline
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)

        Dim pointArray1 As Point3d() = {New Point3d(0.0, 0.0, 0.0), New Point3d(2.0, 10.0, 0.0), New Point3d(5.0, 1.0, 0.0), New Point3d(8.0, 8.0, 0.0)}
        Dim point1 As New Point3dCollection(pointArray1)
        Dim fitTolerance As Double = 0.0000000001
        Dim order As Integer = 2
        spline1 = New Spline(point1, order, fitTolerance)
        Dim pointArray2 As Point3d() = {New Point3d(0.0, 3.0, 0.0), New Point3d(2.0, 6.0, 0.0), New Point3d(9.0, 4.0, 0.0), New Point3d(3.0, 8.0, 0.0)}
        Dim point2 As New Point3dCollection(pointArray2)
        spline2 = New Spline(point2, order, fitTolerance)
        btr.AppendEntity(spline1)
        myT.AddNewlyCreatedDBObject(spline1, True)
        btr.AppendEntity(spline2)
        myT.AddNewlyCreatedDBObject(spline2, True)
        If spline1.HasFitData Then
          spline1.PurgeFitData()
        End If
        If spline2.HasFitData Then
          spline2.PurgeFitData()
        End If

        Dim subentPathEdge1 As FullSubentityPath
        Dim subentPathEdge2 As FullSubentityPath
        Using entity As Entity = DirectCast(myT.GetObject(spline1.ObjectId, OpenMode.ForRead, False), Entity)
          If entity Is Nothing Then
            System.Windows.MessageBox.Show("cannot get entity1")
            Return
          End If
          Dim subentityIdPE As AssocPersSubentityIdPE
          Dim peCls As RXClass = AssocPersSubentityIdPE.GetClass(GetType(AssocPersSubentityIdPE))
          Dim pSubentityIdPE As IntPtr = entity.QueryX(peCls)
          If pSubentityIdPE = IntPtr.Zero Then
            System.Windows.MessageBox.Show("cannot get pSubentityIdPE1")
            Return
          End If
          subentityIdPE = TryCast(AssocPersSubentityIdPE.Create(pSubentityIdPE, False), AssocPersSubentityIdPE)
          If subentityIdPE Is Nothing Then
            System.Windows.MessageBox.Show("cannot get subentityIdPE1")
            Return
          End If
          Dim edgeSubentityIds As SubentityId() = Nothing
          edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge)
          Dim startSID As SubentityId = SubentityId.Null, endSID As SubentityId = SubentityId.Null
          Dim other As SubentityId() = Nothing
          subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds(0), startSID, endSID, other)
          subentPathEdge1 = New FullSubentityPath(New ObjectId(0) {spline1.ObjectId}, edgeSubentityIds(0))
        End Using
        Using entity As Entity = DirectCast(myT.GetObject(spline2.ObjectId, OpenMode.ForRead, False), Entity)
          If entity Is Nothing Then
            System.Windows.MessageBox.Show("cannot get entity2")
            Return
          End If
          Dim subentityIdPE As AssocPersSubentityIdPE
          Dim peCls As RXClass = AssocPersSubentityIdPE.GetClass(GetType(AssocPersSubentityIdPE))
          Dim pSubentityIdPE As IntPtr = entity.QueryX(peCls)
          If pSubentityIdPE = IntPtr.Zero Then
            System.Windows.MessageBox.Show("cannot get pSubentityIdPE2")
            Return
          End If
          subentityIdPE = TryCast(AssocPersSubentityIdPE.Create(pSubentityIdPE, False), AssocPersSubentityIdPE)
          If subentityIdPE Is Nothing Then
            System.Windows.MessageBox.Show("cannot get subentityIdPE2")
            Return
          End If
          Dim edgeSubentityIds As SubentityId() = Nothing
          edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, SubentityType.Edge)
          Dim startSID As SubentityId = SubentityId.Null, endSID As SubentityId = SubentityId.Null
          Dim other As SubentityId() = Nothing
          subentityIdPE.GetEdgeVertexSubentities(entity, edgeSubentityIds(0), startSID, endSID, other)
          subentPathEdge2 = New FullSubentityPath(New ObjectId(0) {spline2.ObjectId}, edgeSubentityIds(0))
        End Using
        Dim consGrpId As ObjectId = GetConstraintGroup(True)

        Dim consGeom1 As ConstrainedCurve = Nothing
        Dim consGeom2 As ConstrainedCurve = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = DirectCast(constGrp.AddConstrainedGeometry(subentPathEdge1), ConstrainedCurve)
          consGeom2 = DirectCast(constGrp.AddConstrainedGeometry(subentPathEdge2), ConstrainedCurve)
          ' Splines are a little different. We have to constrain using 'Implicit Points'.
          ' Query geometry we just constrained for Constrained Implicit Points. First point is the start point.
          Dim consPts1 As ConstrainedImplicitPoint() = consGeom1.ConstrainedImplicitPoints
          Dim consPts2 As ConstrainedImplicitPoint() = consGeom2.ConstrainedImplicitPoints
          constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Smooth, New ConstrainedGeometry() {consPts1(0), consPts2(0)})
        End Using

        myT.Commit()
      End Using
    End Sub




    'Add a symmetric geometrical constraint between two circles, with a line defining the line of symmetry.
    <CommandMethod("TESTSYMMETRY")> _
    Public Shared Sub testSymmetryCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim circle1 As Circle, circle2 As Circle
      Dim line As Line
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        circle1 = New Circle(New Point3d(9, 8, 0), New Vector3d(0, 0, 1), 3)
        circle2 = New Circle(New Point3d(15, 16, 0), New Vector3d(0, 0, 1), 6)
        line = New Line(New Point3d(20, 15, 0), New Point3d(12, 9, 0))
        btr.AppendEntity(circle1)
        myT.AddNewlyCreatedDBObject(circle1, True)
        btr.AppendEntity(circle2)
        myT.AddNewlyCreatedDBObject(circle2, True)
        btr.AppendEntity(line)
        myT.AddNewlyCreatedDBObject(line, True)

        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(circle1, SubentityType.Edge)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(circle2, SubentityType.Edge)
        Dim subentPath3 As FullSubentityPath = GetFullSubentityPath(line, SubentityType.Edge)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Dim consGeom3 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          consGeom3 = constGrp.AddConstrainedGeometry(subentPath3)
          Dim paths As FullSubentityPath() = New FullSubentityPath(2) {subentPath1, subentPath2, subentPath3}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Symmetric, paths)
        End Using

        myT.Commit()
      End Using
    End Sub

    'Add a tangent geometric constraint between a line and a circle
    <CommandMethod("TESTTANGENT")> _
    Public Shared Sub testTangentCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim circle As Circle
      Dim line As Line
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        circle = New Circle(New Point3d(3, 4, 0), New Vector3d(0, 0, 1), 4)
        line = New Line(New Point3d(1, 14, 0), New Point3d(8, -1, 0))
        btr.AppendEntity(circle)
        myT.AddNewlyCreatedDBObject(circle, True)
        btr.AppendEntity(line)
        myT.AddNewlyCreatedDBObject(line, True)

        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(circle, SubentityType.Edge)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(line, SubentityType.Edge)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Tangent, paths)
        End Using

        myT.Commit()
      End Using
    End Sub



    'Add a tangent geometric constraint between a line and a circle
    <CommandMethod("TESTNORMAL")> _
    Public Shared Sub testNormalCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim circle As Circle
      Dim line As Line
      Using myT As Transaction = tm.StartTransaction()
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        circle = New Circle(New Point3d(3, 4, 0), New Vector3d(0, 0, 1), 4)
        line = New Line(New Point3d(1, 14, 0), New Point3d(8, -1, 0))
        btr.AppendEntity(circle)
        myT.AddNewlyCreatedDBObject(circle, True)
        btr.AppendEntity(line)
        myT.AddNewlyCreatedDBObject(line, True)

        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(circle, SubentityType.Edge)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(line, SubentityType.Edge)
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          Dim paths As FullSubentityPath() = New FullSubentityPath(1) {subentPath1, subentPath2}
          Dim newConstraint As GeometricalConstraint = constGrp.AddGeometricalConstraint(GeometricalConstraint.ConstraintType.Normal, paths)
        End Using

        myT.Commit()
      End Using
    End Sub


    <CommandMethod("TESTCREATEVARIABLE")> _
    Public Shared Sub TestCreateVariable()
      AddOrModifyVariable("Side1", 3)
      AddOrModifyVariable("Side2", 4)
      AddOrModifyVariable("_Hypotenuse", "Sqrt(Side1^2+Side2^2)")
    End Sub



    ' Add a radial dimensional constraint to a circle
    <CommandMethod("TESTRADIUS")> _
    Public Shared Sub testRadiusCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim circle As Circle
      Using myT As Transaction = tm.StartTransaction()
        ' Create new circle entity
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        circle = New Circle(New Point3d(25, 25, 0), New Vector3d(0, 0, 1), 3)
        btr.AppendEntity(circle)
        myT.AddNewlyCreatedDBObject(circle, True)
        ' Create new radial dimension entity
        Dim chordPoint As New Point3d(circle.Center.X + circle.Radius, circle.Center.Y, circle.Center.Z)
        Dim dimRadius As New RadialDimension(circle.Center, chordPoint, circle.Radius, "", db.DimStyleTableId)
        btr.AppendEntity(dimRadius)
        myT.AddNewlyCreatedDBObject(dimRadius, True)
        Dim varId As ObjectId, valDepId As ObjectId
        'create user parameter
        varId = AddOrModifyVariable("Radius", "2.0")
        ' Create value dependency (the relationship between the AssocAction and a scalar value - the variable value)
        Using valDep As New AssocValueDependency()
          valDepId = db.AddDBObject(valDep)
          Dim compoundId As New CompoundObjectId(varId, db)
          valDep.AttachToObject(compoundId)
        End Using
        ' Create dim dependency (the relationship between the AssocAction and an Entity)
        Dim dimDepId As ObjectId = ObjectId.Null
        Dim dimDepBodyId As ObjectId = ObjectId.Null
        AssocDimDependencyBody.CreateAndPostToDatabase(dimRadius.ObjectId, dimDepId, dimDepBodyId)
        ' IF adding constraint fails, we want to keep our dimension. We reset to the old value later
        Dim bPreviousValue As Boolean = AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(False)
        ' Add dimensional constraint to the circle
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          ' Get FullSubentityPath of circle edge.
          Dim subentPath As FullSubentityPath = GetFullSubentityPath(circle, SubentityType.Edge)
          ' Constrain circle edge
          consGeom = constGrp.AddConstrainedGeometry(subentPath)
          ' Add constraint to constraint group
          constGrp.AddRadiusDiameterConstraint(consGeom, RadiusDiameterConstraint.RadDiaConstrType.CircleRadius, valDepId, dimDepId)
          ' Reset SetEraseDimensionIfDependencyIsErased back to old value
          AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(bPreviousValue)
        End Using
        myT.Commit()
      End Using
    End Sub



    ' Add a radial dimensional constraint to a circle
    <CommandMethod("TESTRADIUS2")> _
    Public Shared Sub testRadius2Command()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim circle As Circle
      Using myT As Transaction = tm.StartTransaction()
        ' Create new circle entity
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        circle = New Circle(New Point3d(25, 25, 0), New Vector3d(0, 0, 1), 3)
        btr.AppendEntity(circle)
        myT.AddNewlyCreatedDBObject(circle, True)
        ' Get FullSubentityPath of circle edge.
        Dim subentPath As FullSubentityPath = GetFullSubentityPath(circle, SubentityType.Edge)
        Dim varId As ObjectId, valDepId As ObjectId
        'create user parameter
        varId = AddOrModifyVariable("*Radius2", "2.0")
        ' Create value dependency (the relationship between the AssocAction and a scalar value - the variable value)
        Using valDep As New AssocValueDependency()
          valDepId = db.AddDBObject(valDep)
          Dim compoundId As New CompoundObjectId(varId, db)
          valDep.AttachToObject(compoundId)
        End Using
        ' Add dimensional constraint to the circle
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          ' Constrain circle edge
          consGeom = constGrp.AddConstrainedGeometry(subentPath)
          ' Add constraint to constraint group
          constGrp.AddRadiusDiameterConstraint(consGeom, RadiusDiameterConstraint.RadDiaConstrType.CircleRadius, valDepId, ObjectId.Null)
        End Using
        myT.Commit()
      End Using
    End Sub




    'Add a diameter dimensional constraint to a circle
    <CommandMethod("TESTDIAMETER")> _
    Public Shared Sub testDiameterCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim circle As Circle
      Using myT As Transaction = tm.StartTransaction()
        ' Circle entity
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        circle = New Circle(New Point3d(25, 25, 0), New Vector3d(0, 0, 1), 10)
        btr.AppendEntity(circle)
        myT.AddNewlyCreatedDBObject(circle, True)
        ' Diameter Dimension entity
        Dim chordPoint As New Point3d(circle.Center.X - circle.Radius, circle.Center.Y, circle.Center.Z)
        Dim farChordPoint As New Point3d(circle.Center.X + circle.Radius, circle.Center.Y, circle.Center.Z)
        Dim dimDiameter As New DiametricDimension(chordPoint, farChordPoint, circle.Diameter, "", db.DimStyleTableId)
        btr.AppendEntity(dimDiameter)
        myT.AddNewlyCreatedDBObject(dimDiameter, True)

        Dim subentPath As FullSubentityPath = GetFullSubentityPath(circle, SubentityType.Edge)

        Dim idConstrGroup As ObjectId = ObjectId.Null
        ' Create value dependency
        Dim varId As ObjectId, valDepId As ObjectId
        ' Create variable
        varId = AddOrModifyVariable("Diam", "8")
        Using valDep As New AssocValueDependency()
          valDepId = db.AddDBObject(valDep)
          Dim compoundId As New CompoundObjectId(varId, db)
          valDep.AttachToObject(compoundId)
        End Using
        ' Create dim dependency (the relationship between the AssocAction and an Entity)
        Dim dimDepId As ObjectId = ObjectId.Null
        Dim dimDepBodyId As ObjectId = ObjectId.Null
        AssocDimDependencyBody.CreateAndPostToDatabase(dimDiameter.ObjectId, dimDepId, dimDepBodyId)

        Dim bPreviousValue As Boolean = AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(False)

        ' Add dimensional constraint to the entity
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom = constGrp.AddConstrainedGeometry(subentPath)
          constGrp.AddRadiusDiameterConstraint(consGeom, RadiusDiameterConstraint.RadDiaConstrType.CircleRadius, valDepId, dimDepId)
          AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(bPreviousValue)
        End Using
        myT.Commit()
      End Using
    End Sub

    'Add an angular constraint between two lines that depends on two variable parameters
    <CommandMethod("TESTANGULAR")> _
    Public Shared Sub testAngularCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim line1 As Line
      Dim line2 As Line
      Using myT As Transaction = tm.StartTransaction()
        ' Line entities
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        line1 = New Line(New Point3d(10, 0, 0), New Point3d(10, 10, 0))
        line2 = New Line(New Point3d(20, 0, 0), New Point3d(30, 10, 0))
        btr.AppendEntity(line1)
        myT.AddNewlyCreatedDBObject(line1, True)
        btr.AppendEntity(line2)
        myT.AddNewlyCreatedDBObject(line2, True)


        Dim dimAngular As New LineAngularDimension2(line2.StartPoint, line2.EndPoint, line1.StartPoint, line1.EndPoint, New Point3d(27.106, 19.6284, 0), "", _
         db.DimStyleTableId)
        btr.AppendEntity(dimAngular)
        myT.AddNewlyCreatedDBObject(dimAngular, True)

        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(line1, SubentityType.Edge)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(line2, SubentityType.Edge)

        Dim idConstrGroup As ObjectId = ObjectId.Null
        Dim varId As ObjectId, valDepId As ObjectId
        'create user variable 'A'
        AddOrModifyVariable("A", "30")
        'create user variable 'B'
        AddOrModifyVariable("B", "60")
        'create variable 'Angle', dependent on 'A' and 'B'
        varId = AddOrModifyVariable("Angle", "A+B")
        'create value dependency (required to associatevariable with dimensional constraint)
        Using valDep As New AssocValueDependency()
          valDepId = db.AddDBObject(valDep)
          myT.AddNewlyCreatedDBObject(valDep, True)
          Dim compoundId As New CompoundObjectId(varId, db)
          valDep.AttachToObject(compoundId)
        End Using

        'create dim dependency
        Dim dimDepId As ObjectId = ObjectId.Null
        Dim dimDepBodyId As ObjectId = ObjectId.Null
        AssocDimDependencyBody.CreateAndPostToDatabase(dimAngular.ObjectId, dimDepId, dimDepBodyId)

        Dim bPreviousValue As Boolean = AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(False)

        'add dimconstraint to the entities
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          constGrp.AddAngularConstraint(DirectCast(consGeom1, ConstrainedLine), DirectCast(consGeom2, ConstrainedLine), AngularConstraint.AngularSectorType.ParallelClockwise, valDepId, dimDepId)
          AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(bPreviousValue)
        End Using
        myT.Commit()
      End Using
    End Sub



    'Add an angular constraint to an arc, but dimensional constraint isn't linked to a dimension entity
    <CommandMethod("TESTANGULAR2")> _
    Public Shared Sub testAngular2Command()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Dim arc1 As Arc
      Using myT As Transaction = tm.StartTransaction()
        ' Create Arc entity
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        arc1 = New Arc(New Point3d(10, 10, 0), 5, 0, Math.PI / 3)
        btr.AppendEntity(arc1)
        myT.AddNewlyCreatedDBObject(arc1, True)
        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(arc1, SubentityType.Edge)
        Dim varId As ObjectId, valDepId As ObjectId
        'create variable 'Angle'
        varId = AddOrModifyVariable("Angle2", "60")
        'create value dependency (required to associate variable with dimensional constraint)
        Using valDep As New AssocValueDependency()
          valDepId = db.AddDBObject(valDep)
          myT.AddNewlyCreatedDBObject(valDep, True)
          Dim compoundId As New CompoundObjectId(varId, db)
          valDep.AttachToObject(compoundId)
        End Using
        'add dimconstraint to the entities
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedCurve = Nothing
        Dim startPt As ConstrainedPoint = Nothing
        Dim endPt As ConstrainedPoint = Nothing
        Dim centerPt As ConstrainedPoint = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = DirectCast(constGrp.AddConstrainedGeometry(subentPath1), ConstrainedCurve)
          Dim consPts As ConstrainedImplicitPoint() = consGeom1.ConstrainedImplicitPoints

          For Each pt As ConstrainedImplicitPoint In consPts
            If pt.PointType = ConstrainedImplicitPoint.ImplicitPointType.StartImplicit Then
              startPt = pt
            End If
            If pt.PointType = ConstrainedImplicitPoint.ImplicitPointType.EndImplicit Then
              endPt = pt
            End If
            If pt.PointType = ConstrainedImplicitPoint.ImplicitPointType.CenterImplicit Then
              centerPt = pt
            End If
          Next
          constGrp.Add3PointAngularConstraint(centerPt, startPt, endPt, AngularConstraint.AngularSectorType.ParallelClockwise, valDepId, ObjectId.Null)
        End Using

        myT.Commit()
      End Using
    End Sub



    'Add an aligned dimensional constraint between two DBPoints
    <CommandMethod("TEST2POINTDISTANCE")> _
    Public Shared Sub test2PointDistanceCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Using myT As Transaction = tm.StartTransaction()
        ' Point entities
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        Dim point3d1 As New Point3d(10, 10, 0)
        Dim point3d2 As New Point3d(20, 20, 0)
        Dim dimPt As New Point3d(36, 20, 0)
        Dim point1 As New DBPoint(point3d1)
        Dim point2 As New DBPoint(point3d2)
        btr.AppendEntity(point1)
        btr.AppendEntity(point2)
        myT.AddNewlyCreatedDBObject(point1, True)
        myT.AddNewlyCreatedDBObject(point2, True)

        ' Aligned Dimension entity
        Dim dimAligned As New AlignedDimension(point3d1, point3d2, dimPt, "ALIGNED DIMENSION", db.Dimstyle)
        btr.AppendEntity(dimAligned)
        myT.AddNewlyCreatedDBObject(dimAligned, True)


        Dim subentPath1 As FullSubentityPath = GetFullSubentityPath(point1, SubentityType.Vertex)
        Dim subentPath2 As FullSubentityPath = GetFullSubentityPath(point2, SubentityType.Vertex)

        Dim idConstrGroup As ObjectId = ObjectId.Null
        'create value dependency
        Dim varId As ObjectId, valDepId As ObjectId
        varId = AddOrModifyVariable("Dist", "10.0")

        Using valDep As New AssocValueDependency()
          valDepId = db.AddDBObject(valDep)
          Dim compoundId As New CompoundObjectId(varId, db)
          valDep.AttachToObject(compoundId)
        End Using

        'create dim dependency
        Dim dimDepId As ObjectId = ObjectId.Null
        Dim dimDepBodyId As ObjectId = ObjectId.Null
        AssocDimDependencyBody.CreateAndPostToDatabase(dimAligned.ObjectId, dimDepId, dimDepBodyId)

        Dim bPreviousValue As Boolean = AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(False)

        'add dimconstraint to the entities
        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom1 As ConstrainedGeometry = Nothing
        Dim consGeom2 As ConstrainedGeometry = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom1 = constGrp.AddConstrainedGeometry(subentPath1)
          consGeom2 = constGrp.AddConstrainedGeometry(subentPath2)
          constGrp.AddDistanceConstraint(consGeom1, consGeom2, DistanceConstraint.DistanceDirectionType.NotDirected, valDepId, dimDepId, New Vector3d(1, 0, 0), _
           Nothing)
          AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(bPreviousValue)
        End Using
        myT.Commit()
      End Using
    End Sub






    'Add an aligned dimensional constraint to a Line.
    <CommandMethod("TESTALIGNEDDIM")> _
    Public Shared Sub testAlignedDimCommand()
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Using myT As Transaction = tm.StartTransaction()
        ' Point entities
        Dim bt As BlockTable = DirectCast(myT.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
        Dim btr As BlockTableRecord = DirectCast(myT.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
        Dim point3d1 As New Point3d(10, 10, 0)
        Dim point3d2 As New Point3d(20, 20, 0)
        Dim dimPt As New Point3d(36, 20, 0)
        Dim line1 As New Line(point3d1, point3d2)
        btr.AppendEntity(line1)
        myT.AddNewlyCreatedDBObject(line1, True)

        ' Aligned Dimension entity
        Dim dimAligned As New AlignedDimension(point3d1, point3d2, dimPt, "ALIGNED DIMENSION", db.Dimstyle)
        btr.AppendEntity(dimAligned)
        myT.AddNewlyCreatedDBObject(dimAligned, True)

        Dim idConstrGroup As ObjectId = ObjectId.Null

        'To query the subentities of the line, we create and use a protocol extension (PE) provided by the associativity API
        Dim subentityIdPE As AssocPersSubentityIdPE
        Dim peCls As RXClass = AssocPersSubentityIdPE.GetClass(GetType(AssocPersSubentityIdPE))
        Dim pSubentityIdPE As IntPtr = line1.QueryX(peCls)
        If pSubentityIdPE = IntPtr.Zero Then
          System.Windows.MessageBox.Show("cannot get pSubentityIdPE")
          Return
        End If
        subentityIdPE = TryCast(AssocPersSubentityIdPE.Create(pSubentityIdPE, False), AssocPersSubentityIdPE)
        If subentityIdPE Is Nothing Then
          System.Windows.MessageBox.Show("cannot get subentityIdPE")
          Return
        End If
        'Now we have the PE, we query the subentities
        Dim edgeSubentityIds As SubentityId() = Nothing
        'First we retrieve a list of all edges (a line has one edge)
        edgeSubentityIds = subentityIdPE.GetAllSubentities(line1, SubentityType.Edge)
        Dim startSID As SubentityId = SubentityId.Null, endSID As SubentityId = SubentityId.Null
        Dim other As SubentityId() = Nothing
        'Now we retrieve the vertices associated with that edge.
        subentityIdPE.GetEdgeVertexSubentities(line1, edgeSubentityIds(0), startSID, endSID, other)
        'The PE returns a SubEntId. We want a Full SubentityPath
        Dim subentPathEdge As New FullSubentityPath(New ObjectId(0) {line1.ObjectId}, edgeSubentityIds(0))
        Dim subentPath1 As New FullSubentityPath(New ObjectId(0) {line1.ObjectId}, startSID)
        Dim subentPath2 As New FullSubentityPath(New ObjectId(0) {line1.ObjectId}, endSID)

        'add dimconstraint to the entities

        'create value dependency
        Dim varId As ObjectId, valDepId As ObjectId
        varId = AddOrModifyVariable("Aligned", "10.0")

        Using valDep As New AssocValueDependency()
          valDepId = db.AddDBObject(valDep)
          Dim compoundId As New CompoundObjectId(varId, db)
          valDep.AttachToObject(compoundId)
        End Using

        'create dim dependency
        Dim dimDepId As ObjectId = ObjectId.Null
        Dim dimDepBodyId As ObjectId = ObjectId.Null
        AssocDimDependencyBody.CreateAndPostToDatabase(dimAligned.ObjectId, dimDepId, dimDepBodyId)

        Dim bPreviousValue As Boolean = AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(False)

        Dim consGrpId As ObjectId = GetConstraintGroup(True)
        Dim consGeom0 As ConstrainedGeometry = Nothing
        Dim consLine As ConstrainedLine = Nothing
        Using constGrp As Assoc2dConstraintGroup = DirectCast(myT.GetObject(consGrpId, OpenMode.ForWrite, False), Assoc2dConstraintGroup)
          consGeom0 = constGrp.AddConstrainedGeometry(subentPathEdge)

          'Sometimes you have to access implicit constrained geometry. 
          'In this case, we're constraining the lines by its endpoints, so we have to query the constrained line edge for its ConstrainedImplicitPoints.
          'If we try to pass the line's end point subentity directly via a call to AddConstrainedGeometry, we'll get an error.
          consLine = DirectCast(consGeom0, ConstrainedLine)
          Dim consPts As ConstrainedImplicitPoint() = consLine.ConstrainedImplicitPoints
          Dim consStartPt As ConstrainedGeometry = Nothing
          Dim consEndPt As ConstrainedGeometry = Nothing
          'We don't know which is start or end point, so we test. (And we could have a midpoint in there too).
          For Each pt As ConstrainedImplicitPoint In consPts
            If pt.PointType = ConstrainedImplicitPoint.ImplicitPointType.StartImplicit Then
              consStartPt = pt
            End If
            If pt.PointType = ConstrainedImplicitPoint.ImplicitPointType.EndImplicit Then
              consEndPt = pt

            End If
          Next
          'If something went wrong ...
          If (consStartPt Is Nothing) OrElse (consEndPt Is Nothing) Then
            Return
          End If

          'Now we add our constraint
          constGrp.AddDistanceConstraint(consStartPt, consEndPt, DistanceConstraint.DistanceDirectionType.NotDirected, valDepId, dimDepId, New Vector3d(1, 0, 0), _
           Nothing)
          AssocDimDependencyBodyBase.SetEraseDimensionIfDependencyIsErased(bPreviousValue)
        End Using
        myT.Commit()
      End Using
    End Sub


 

    ' Helper function to return FullSubEntPath for subent of provided entity
    Private Shared Function GetFullSubentityPath(ByVal entity As Entity, ByVal subentityType As SubentityType) As FullSubentityPath
      If entity Is Nothing Then
        System.Windows.MessageBox.Show("cannot get entity")
        Return New FullSubentityPath()
      End If
      Dim subentityIdPE As AssocPersSubentityIdPE
      Dim peCls As RXClass = AssocPersSubentityIdPE.GetClass(GetType(AssocPersSubentityIdPE))
      Dim pSubentityIdPE As IntPtr = entity.QueryX(peCls)
      If pSubentityIdPE = IntPtr.Zero Then
        System.Windows.MessageBox.Show("cannot get pSubentityIdPE")
        Return New FullSubentityPath()
      End If
      subentityIdPE = TryCast(AssocPersSubentityIdPE.Create(pSubentityIdPE, False), AssocPersSubentityIdPE)
      If subentityIdPE Is Nothing Then
        System.Windows.MessageBox.Show("cannot get subentityIdPE")
        Return New FullSubentityPath()
      End If
      Dim edgeSubentityIds As SubentityId() = Nothing
      edgeSubentityIds = subentityIdPE.GetAllSubentities(entity, subentityType)
      Return New FullSubentityPath(New ObjectId(0) {entity.ObjectId}, edgeSubentityIds(0))
    End Function




#Region "Helper functions"


    ' Helper function to retrieve (or create) constraint group
    Private Shared Function GetConstraintGroup(ByVal createIfDoesNotExist As Boolean) As ObjectId
      ' Calculate the current plane on which new entities are added by the editor
      ' (A combination of UCS and ELEVATION sysvar).
      Dim ed As Autodesk.AutoCAD.EditorInput.Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
      Dim ucsMatrix As Matrix3d = ed.CurrentUserCoordinateSystem
      Dim origin As Point3d = ucsMatrix.CoordinateSystem3d.Origin
      Dim xAxis As Vector3d = ucsMatrix.CoordinateSystem3d.Xaxis
      Dim yAxis As Vector3d = ucsMatrix.CoordinateSystem3d.Yaxis
      Dim zAxis As Vector3d = ucsMatrix.CoordinateSystem3d.Zaxis
      origin = origin + CDbl(Application.GetSystemVariable("ELEVATION")) * zAxis
      Dim currentPlane As New Plane(origin, xAxis, yAxis)

      ' get the constraint group from block table record
      Dim idConstrGroup As ObjectId = ObjectId.Null
      Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
      Dim networkId As ObjectId = AssocNetwork.GetInstanceFromObject(SymbolUtilityServices.GetBlockModelSpaceId(db), createIfDoesNotExist, True, "")
      If networkId.IsNull Then
        System.Windows.MessageBox.Show("network id is null")
        Return idConstrGroup
      End If

      ' Try to find the constraint group in the associative network
      Dim tm As Autodesk.AutoCAD.DatabaseServices.TransactionManager = db.TransactionManager
      Using myT As Transaction = tm.StartTransaction()
        Using network As AssocNetwork = DirectCast(myT.GetObject(networkId, OpenMode.ForRead, False), AssocNetwork)
          If network Is Nothing Then
            Return idConstrGroup
          End If
          ' Iterate all actions in network to find Assoc2dConstraintGroups
          Dim actionsInNetwork As ObjectIdCollection = network.GetActions
          For nCount As Integer = 0 To actionsInNetwork.Count - 1
            Dim idAction As ObjectId = actionsInNetwork(nCount)
            If idAction = ObjectId.Null Then
              Continue For
            End If
            ' Is this action a type of Assoc2dConstraintGroup?
            If idAction.ObjectClass.IsDerivedFrom(RXObject.GetClass(GetType(Autodesk.AutoCAD.DatabaseServices.Assoc2dConstraintGroup))) Then
              Using action As AssocAction = DirectCast(myT.GetObject(idAction, OpenMode.ForRead, False), AssocAction)
                If action Is Nothing Then
                  Continue For
                End If
                Dim constGrp As Assoc2dConstraintGroup = DirectCast(action, Assoc2dConstraintGroup)
                ' Is this the Assoc2dConstraintGroup for our plane of interest?
                If constGrp.GetWorkPlane.IsCoplanarTo(currentPlane) Then
                  ' If it is then we've found an existing constraint group we can use.
                  Return idAction
                End If
              End Using
            End If
          Next
        End Using

        ' If we get to here, a suitable contraint group doesn't exist, create a new one if that's what calling fn wanted.
        If idConstrGroup.IsNull AndAlso createIfDoesNotExist Then
          Using network As AssocNetwork = DirectCast(myT.GetObject(networkId, OpenMode.ForWrite, False), AssocNetwork)
            ' Create  construction plane
            Dim constraintPlane As New Plane(currentPlane)
            ' If model extent is far far away from origin then we need to shift 
            '  construction plane origin within the model extent.
            ' (Use Pextmin, PExtmax in paper space)
            Dim extmin As Point3d = db.Extmin
            Dim extmax As Point3d = db.Extmax
            If extmin.GetAsVector().Length > 100000000.0 Then
              Dim originL As Point3d = extmin + (extmax - extmin) / 2.0
              Dim result As PointOnSurface = currentPlane.GetClosestPointTo(originL)
              constraintPlane.[Set](result.GetPoint(), currentPlane.Normal)
            End If
            ' Create the new constraint group and add it to the associative network.
            Using constGrp As New Assoc2dConstraintGroup(constraintPlane)
              idConstrGroup = db.AddDBObject(constGrp)
            End Using
            network.AddAction(idConstrGroup, True)
          End Using
        End If
        myT.Commit()
      End Using
      Return idConstrGroup
    End Function



    ' Helper function to add a new variable to the associative network or modify expression of the existing one
    Private Shared Function AddOrModifyVariable(ByVal varName As String, ByVal varExpression As String) As ObjectId
      Dim varId As ObjectId = ObjectId.Null
      Dim doc As Document = Application.DocumentManager.MdiActiveDocument
      Dim db As Database = doc.Database
      Using myT As Transaction = db.TransactionManager.StartTransaction()
        ' Open the AssocNetwork
        Dim networkId As ObjectId = AssocNetwork.GetInstanceFromObject(SymbolUtilityServices.GetBlockModelSpaceId(db), True, True, "")
        Dim network As AssocNetwork = DirectCast(myT.GetObject(networkId, OpenMode.ForWrite), AssocNetwork)

        Dim var As AssocVariable = Nothing
        ' Iterate through all actions in the network
        Dim actionIds As ObjectIdCollection = network.GetActions
        For Each actionId As ObjectId In network.GetActions
          ' Is this action an AssocVariable?
          If actionId.ObjectClass.IsDerivedFrom(RXObject.GetClass(GetType(Autodesk.AutoCAD.DatabaseServices.AssocVariable))) Then
            ' If so, we check if it has the name we're looking for.
            var = DirectCast(myT.GetObject(actionId, OpenMode.ForWrite), AssocVariable)
            If var IsNot Nothing Then
              ' If name matches, then we exit loop and set its expression
              If var.Name = varName Then
                Exit For
              Else
                var = Nothing
              End If
            End If
          End If
        Next
        ' If variable with correct name wasn't found, we create it.
        If var Is Nothing Then
          var = New AssocVariable()
          varId = network.Database.AddDBObject(var)
          network.AddAction(varId, True)
          myT.AddNewlyCreatedDBObject(var, True)
          var.SetName(varName, True)
        End If
        ' Finally we set its expression to the new value.
        Dim errMsg As String = ""
        var.SetExpression(varExpression, "", True, True, errMsg, False)
        Dim rb As New ResultBuffer()
        errMsg = var.EvaluateExpression(rb)
        var.Value = rb
        myT.Commit()
      End Using
      Return varId
    End Function





    ' Helper function to handle exceptions being thrown if you try to add the same constrained geometry twice.
    ' You could work out what constrained geometry to add up front. That would be more efficient as no exceptions would be thrown,
    '  but isn't always possible (e.g. if running a command multiple times to constrain existing geometry in a drawing.
    ' (This helper function isn't used by any functions in this sample project).
    Private Shared Function AddConstrainedGeometry(ByVal constGrp As Assoc2dConstraintGroup, ByVal path As FullSubentityPath) As ConstrainedGeometry
      Dim consGeom As ConstrainedGeometry = Nothing
      Try
        consGeom = constGrp.AddConstrainedGeometry(path)
      Catch ex As Autodesk.AutoCAD.Runtime.Exception
        ' This error isn't really an an error
        If ex.ErrorStatus <> ErrorStatus.AlreadyInGroup Then
          Throw ex
        End If
        ' If we get to here, then 'path' was already in the group so we can retrieve it.
        consGeom = constGrp.GetConstrainedGeometry(path, False)
      End Try
      Return consGeom
    End Function


#End Region


  End Class

End Namespace